@@ -1,12 +1,13 @@
-/* GNU/Linux/ARC specific low level interface, for the remote server for GDB.
-   Copyright 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004
-   Free Software Foundation, Inc.
+/* Target dependent code for the remote server for GNU/Linux ARC.
+
+   Copyright 2014-2016 Free Software Foundation, Inc.
+   Contributed by Synopsys Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,20 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   --------------------------------------------------------------------------
-
-   The comments within this file are also licensed under under the terms of
-   the GNU Free Documentation License as published by the Free Software
-   Foundation; either version 1.3 of the License, or (at your option) any
-   later version. See the file fdi.texi in the gdb/doc directory for copying
-   conditions.
-
-   You should have received a copy of the GNU Free Documentation License along
-   with this program. If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* -------------------------------------------------------------------------- */
 /*!@file
@@ -54,8 +42,6 @@
 
 #include <linux/elf.h>
 #include <arpa/inet.h>
-#include <asm/ptrace.h>
-#include <sys/ptrace.h>
 
 /* From include/uapi/asm/ptrace.h in the ARC Linux sources.
  * See user_regs_struct.
@@ -314,7 +300,7 @@ arc_breakpoint_at (CORE_ADDR where)
 static void
 arc_fill_gregset (struct regcache *regcache, void *buf)
 {
-  struct user_regs_struct *regbuf = buf;
+  struct user_regs_struct *regbuf = (struct user_regs_struct *) buf;
 
   /* Collect all the unique registers by name. */
   collect_register_by_name (regcache, "r0", &(regbuf->scratch.r0));
@@ -398,7 +384,7 @@ arc_fill_gregset (struct regcache *regcache, void *buf)
 static void
 arc_store_gregset (struct regcache *regcache, const void *buf)
 {
-  const struct user_regs_struct *regbuf = buf;
+  const struct user_regs_struct *regbuf = (const struct user_regs_struct *) buf;
 
   /* Supply all the unique registers by name. */
   supply_register_by_name (regcache, "r0", &(regbuf->scratch.r0));
@@ -470,7 +456,7 @@ arc_store_gregset (struct regcache *regcache, const void *buf)
  *
  * This is the same function as in arc-linux-nat.c */
 ps_err_e
-ps_get_thread_area (const struct ps_prochandle *ph, lwpid_t lwpid, int idx,
+ps_get_thread_area (struct ps_prochandle *ph, lwpid_t lwpid, int idx,
                     void **base)
 {
     if (ptrace (PTRACE_GET_THREAD_AREA, lwpid, NULL, base) != 0)
@@ -506,13 +492,7 @@ static struct regset_info arc_regsets[] =
     .type           = GENERAL_REGS,
     .fill_function  = arc_fill_gregset,
     .store_function = arc_store_gregset },
-  { .get_request    = 0,		/* End marker */
-    .set_request    = 0,
-    .nt_type        = 0,
-    .size           = -1,
-    .type           = -1,
-    .fill_function  = NULL,
-    .store_function = NULL }
+  NULL_REGSET
 };
 
 static struct regsets_info arc_regsets_info =
@@ -559,34 +539,18 @@ arc_regs_info (void)
            for single step, so will set breakpoints by using memory
            read/write. So we do not need the breakpoint functions. */
 struct linux_target_ops the_low_target = {
-  .arch_setup                       = arc_arch_setup,
-  .regs_info                        = arc_regs_info,
-  .cannot_fetch_register            = arc_cannot_fetch_register,
-  .cannot_store_register            = arc_cannot_store_register,
-  .fetch_register                   = NULL,
-  .get_pc                           = arc_get_pc,
-  .set_pc                           = arc_set_pc,
-  .breakpoint                       = NULL,
-  .breakpoint_len                   = 2,
-  .breakpoint_reinsert_addr         = NULL,
-  .decr_pc_after_break              = 0,
-  .breakpoint_at                    = arc_breakpoint_at,
-  .insert_point                     = NULL,
-  .remove_point                     = NULL,
-  .stopped_by_watchpoint            = NULL,
-  .stopped_data_address             = NULL,
-  .collect_ptrace_register          = NULL,
-  .supply_ptrace_register           = NULL,
-  .siginfo_fixup                    = NULL,
-  .new_process                      = NULL,
-  .new_thread                       = NULL,
-  .prepare_to_resume                = NULL,
-  .process_qsupported               = NULL,
-  .supports_tracepoints             = NULL,
-  .get_thread_area                  = NULL,
-  .install_fast_tracepoint_jump_pad = NULL,
-  .emit_ops                         = NULL,
-  .get_min_fast_tracepoint_insn_len = NULL
+  arc_arch_setup,
+  arc_regs_info,
+  arc_cannot_fetch_register,
+  arc_cannot_store_register,
+  NULL, /* fetch_register  */
+  arc_get_pc,
+  arc_set_pc,
+  NULL, /* breakpoint_kind_from_pc  */
+  NULL, /* sw_breakpoint_from_kind  */
+  NULL, /* get_next_pcs  */
+  0, /* .decr_pc_after_break  */
+  arc_breakpoint_at,
 };
 
 void
@@ -596,3 +560,5 @@ initialize_low_arch (void)
 
   initialize_regsets_info (&arc_regsets_info);
 }
+
+/* vim: set sts=2 shiftwidth=2 ts=8: */
